"""

Gnome Python Tools: GnomeFileSelection, GnomeProgressbar, GnomePixmap, GnomePixmapWithLabel, 
                    GnomePopupQuestionDialog, GnomeOptionMenu

"""

from os import environ

if (not environ.has_key("DISPLAY")):              # This happens when started in a console
    raise ImportError, "DISPLAY environment variable not set"

import gtk

from time import sleep
from os import listdir, stat, getcwd, chmod
from string import join, split, lower, upper, zfill, strip, atoi, find
from time import localtime, asctime, time
from ConfigParser import ConfigParser
from re import sub
from tempfile import mktemp
from whrandom import choice
from ConfigParser import ConfigParser

try:
    from log4py import Logger
    log4pyavailable = gtk.TRUE
except ImportError:
    log4pyavailable = gtk.FALSE

import os

# Dicitionary caching common used GdkColors
GdkColorCache = {}

# Drag & Drop target definition
targets = [('text/plain', 0, 1),
            ('text/uri-list', 0, 2),
            ('STRING', 0, 3)]

# Constants for various file-types
ALL_FILES = ("All Files", ["*"])
AUDIO_FILES = ("Audio Files", ["mp3", "ogg", "wav"])
EROASTER_PROJECTS = ("ERoaster Projects", ["erp"])
ISO_IMAGES = ("ISO Images", ["iso", "img"])
XMMS_PLAYLISTS = ("XMMS Playlists", ["m3u"])
WINGIDE_PROJECTS = ("WingIDE Projects", ["wpr"])
CUE_SHEETS = ("Cuesheets", ["cue", "CUE"])
BIN_FILES = ("BIN files", ["bin", "BIN"])

# Short cuts (default)
SHORTCUTS = [ [ "HOME", "$HOME" ],
              [ "DESKTOP", "$DESKTOP" ],
              [ "START", "$CWD" ],
              [ "LASTDIR", "$LASTDIR"],
              [ "", ""],
              [ "", ""]
              ]

# Dictionary containing icons for different filetypes
ICONS = {}
ICONS["DIRECTORY"] = "folder.xpm"                      # DO NOT CHANGE THIS ONE !
ICONS["UNKNOWN"] = "unknown.xpm"                       # DO NOT CHANGE THIS ONE !

ICONS["HOME"] = "home.xpm"                             # These icons are for the shortcuts
ICONS["DESKTOP"] = "desktop.xpm"
ICONS["START"] = "start.xpm"
ICONS["LASTDIR"] = "lastdir.xpm"
ICONS["CDROM"] = "cdrom.xpm"
ICONS["HARDDISK"] = "harddisk.xpm"
ICONS["NETWORK"] = "network.xpm"
ICONS["LINK"] = "link.xpm"

ICONS["DOC"] = "doc.xpm"                               # These icons are compared to file extensions
ICONS["GZ"] = "compressed.xpm"
ICONS["BZ2"] = "compressed.xpm"
ICONS["GIF"] = "image.xpm"
ICONS["JPG"] = "image.xpm"
ICONS["BMP"] = "image.xpm"
ICONS["PNG"] = "image.xpm"
ICONS["XPM"] = "image.xpm"
ICONS["PATCH"] = "patch.xpm"
ICONS["PS"] = "ps.xpm"
ICONS["MP3"] = "sound.xpm"
ICONS["WAV"] = "sound.xpm"
ICONS["OGG"] = "sound.xpm"
ICONS["TAR"] = "tar.xpm"
ICONS["TXT"] = "text.xpm"

def create_pixmap(widget, xpm):
    return gtk.gdk.pixmap_colormap_create_from_xpm(None, widget.get_colormap(), None, xpm)

# Returns a string representation of a number (with a pretty-print option)
def lng2str(number, prettyprint = gtk.TRUE):
    if (prettyprint == gtk.TRUE):
        number = list(str(number))
        if number[-1] == "L":
            number = number[:-1]
        number.reverse()
        dots = divmod(len(number) - 1, 3)[0]
        for i in range(dots):
            number.insert(3 + (i * 3) + i, ".")
        number.reverse()
        return join(number, "")
    else:
        return str(number)

# Converts RGB value to a GdkColor
def hex2gdkcolor(widget, color):
    global GdkColorCache
    if (color[0] == "#"):
        color = color[1:]
    if (GdkColorCache.has_key(color)):
        return GdkColorCache[color]
    else:
        red = eval("0x%s" % color[0:2]) * 257
        green = eval("0x%s" % color[2:4]) * 257
        blue = eval("0x%s" % color[4:6]) * 257
        color_map = widget.get_colormap()
        gdkcolor = color_map.alloc_color(red, green, blue)
        if (not GdkColorCache.has_key(color)):
            GdkColorCache[color] = gdkcolor
        return gdkcolor

def indexof(dict, elem):
    for i in range(len(dict)):
        if (dict[i] == elem):
            return i

def get_file_icon(filename, window, icondir):
    if (os.path.islink(filename)):
        filename = "LINK"
    elif (os.path.isdir(filename)):
        filename = "DIRECTORY"
    if (filename in [ "LINK", "DIRECTORY", "HOME", "DESKTOP", "START", "LASTDIR", "CDROM", "HARDDISK", "NETWORK" ]):
        xpmname = ICONS[filename]
    else:
        extension = upper(os.path.splitext(filename)[1])
        if (len(extension) > 0):
            extension = extension[1:]
        if (ICONS.has_key(extension)):
            xpmname = ICONS[extension]
        else:
            xpmname = ICONS["UNKNOWN"]
    if (len(icondir) > 0):
        if (icondir[-1] != "/"):
            icondir = "%s/" % icondir
        icon, mask = create_pixmap(window, "%s%s" % (icondir, xpmname))
        return (icon, mask)
    else:
        return None

def format_duration(seconds):
    hours = int(divmod(seconds, (60 * 60))[0])
    seconds = seconds - (hours * (60 * 60))
    minutes = int(divmod(seconds, 60)[0])
    seconds = str(seconds - (minutes * 60))
    splitted = split(seconds, ".")
    seconds = zfill(splitted[0], 2)
    if (len(splitted) > 1):
        milliseconds = splitted[1][:3]
    else:
        milliseconds = ""
    while (len(milliseconds) < 3):
        milliseconds = "%s0" % milliseconds
    return "%s:%s:%s.%s" % (zfill(hours, 2), zfill(minutes, 2), seconds, milliseconds)

class FileSelectionDialog:
    """ Advanced file selection dialog. """

    def __init__(self, path = getcwd(), showhiddenfiles = gtk.FALSE, multiselection = gtk.TRUE, filetypes = [ALL_FILES], icondir = "", windowtitle = "Open File", loglevel = 1 << 1):
        self.filelist = []
        self.path = path
        self.showhiddenfiles = showhiddenfiles
        self.filetypes = filetypes
        self.currentextension = filetypes[0][1]
        self.icondir = icondir
        self.windowtitle = windowtitle
        if (len(self.icondir) > 0):
            if (self.icondir[-1] != "/"):
                self.icondir = "%s/" % self.icondir
        if (multiselection == gtk.TRUE):
            self.selectiontype = gtk.SELECTION_EXTENDED
        else:
            self.selectiontype = gtk.SELECTION_SINGLE
        self.selectedfiles = []
        self.__GnomeFileSelection__lastdirectory = ""
        self.__GnomeFileSelection_lastdirectoryread = ""
        self.__GnomeFileSelection__shortcuts = SHORTCUTS
        if (log4pyavailable == gtk.TRUE):
            self.log4py = Logger().get_instance(self)
            self.log4py.set_loglevel(loglevel)
            self.log4py.debug("GnomeFileSelection initialized")
        else:
            self.log4py = None
        self.__GnomeFileSelection_drawwindow()

    def __GnomeFileSelection_drawwindow(self):

        self.window = gtk.Window()
        self.window.set_title(self.windowtitle)
        self.window.set_border_width(5)
        lookinlabel = gtk.Label("Look in:")
        lookinlabel.show()
        self.lookinpath = gtk.Entry()
        self.lookinpath.connect("key_press_event", self.__GnomeFileSelection_changedirectorymanually)
        self.lookinpath.set_text(self.path)
        self.lookinpath.show()
        self.lookinparent = gtk.Button(" .. ")
        self.lookinparent.connect("clicked", self.__GnomeFileSelection_changetoparentdir)
        self.lookinparent.show()
        lookinbox = gtk.HBox(spacing = 5)
        lookinbox.pack_start(lookinlabel, expand = gtk.FALSE)
        lookinbox.pack_start(self.lookinpath)
        lookinbox.pack_start(self.lookinparent, expand = gtk.FALSE)
        lookinbox.show()

        if (self.icondir != ""):
            self.fileclist = gtk.CList(4, ["", " Name ", " Size ", " Last modified "])
            self.fileclist.set_column_width(0, 18)
            self.fileclist.set_column_width(1, 132)
            self.fileclist.set_column_width(2, 80)
            self.fileclist.set_column_width(3, 155)
            self.fileclist.set_column_justification(2, gtk.JUSTIFY_RIGHT)
            self.fileclist.set_column_justification(3, gtk.JUSTIFY_RIGHT)
        else:
            self.fileclist = gtk.CList(3, [" Name ", " Size ", " Last modified "])
            self.fileclist.set_column_width(0, 150)
            self.fileclist.set_column_width(1, 80)
            self.fileclist.set_column_width(2, 155)
            self.fileclist.set_column_justification(1, gtk.JUSTIFY_RIGHT)
            self.fileclist.set_column_justification(2, gtk.JUSTIFY_RIGHT)
        self.fileclist.set_size_request(440, 245)
        self.fileclist.set_selection_mode(self.selectiontype)
        self.fileclist.connect("key_press_event", self.__GnomeFileSelection_changedirectorykeyboard)
        self.fileclist.connect("button_press_event", self.__GnomeFileSelection_changedirectory)
        self.fileclist.connect("select_row", self.__GnomeFileSelection_updatefileselection)
        self.fileclist.connect("unselect_row", self.__GnomeFileSelection_updatefileselection)
        self.fileclist.show()

        filescrolledwin = gtk.ScrolledWindow()
        filescrolledwin.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        filescrolledwin.add(self.fileclist)
        filescrolledwin.show()

        filenamelabel = gtk.Label("Filename:")
        filenamelabel.set_alignment(0, 0.5)
        filenamelabel.show()
        self.filename = gtk.Entry()
        self.filename.show()

        filetypeoptionmenu = gtk.OptionMenu()
        self.filetypemenu = gtk.Menu()
        self.filetypemenu.show()
        self.filetypeitems = {}
        for i in range(len(self.filetypes)):
            description = self.filetypes[i][0] + " ("
            for j in range(len(self.filetypes[i][1])):
                extension = self.filetypes[i][1][j]
                if (extension == "*"):
                    description = "%s%s; " % (description, extension)
                else:
                    description = "%s*.%s; " % (description, extension)
            description = "%s)" % description[:-2]
            self.filetypeitems[i] = gtk.MenuItem(description)
            self.filetypeitems[i].connect("activate", self.__GnomeFileSelection_activatefiletype)
            self.filetypeitems[i].show()
            self.filetypemenu.append(self.filetypeitems[i])
        self.__GnomeFileSelection__updateonactivate = gtk.FALSE
        self.filetypemenu.activate_item(self.filetypeitems[0], gtk.TRUE)
        self.__GnomeFileSelection__updateonactivate = gtk.TRUE
        filetypeoptionmenu.set_menu(self.filetypemenu)
        filetypeoptionmenu.show()

        filetypelabel = gtk.Label("Files of type: ")
        filetypelabel.set_alignment(0, 0.5)
        filetypelabel.show()

        filetable = gtk.Table(2, 2)
        filetable.attach(filenamelabel, 0, 1, 0, 1, gtk.FILL, 0)
        filetable.attach(self.filename, 1, 2, 0, 1)
        filetable.attach(filetypelabel, 0, 1, 1, 2, gtk.FILL, 0)
        filetable.attach(filetypeoptionmenu, 1, 2, 1, 2, gtk.FILL, 0)
        filetable.show()

        self.hidden = gtk.CheckButton("Show hidden files")
        self.hidden.set_active(self.showhiddenfiles)
        self.hidden.connect("clicked", self.__GnomeFileSelection_changehiddenfiles)
        self.hidden.show()
        dummy = gtk.Label("")
        dummy.show()
        self.ok_button = gtk.Button(stock="gtk-ok")
        self.ok_button.set_size_request(75, -1)
        self.ok_button.show()
        self.close_button = gtk.Button(stock="gtk-close")
        self.close_button.set_size_request(75, -1)
        self.close_button.show()
        buttonbox = gtk.HBox(spacing = 5)
        buttonbox.pack_start(self.hidden, expand = gtk.FALSE)
        buttonbox.pack_start(dummy)
        buttonbox.pack_start(self.ok_button, expand = gtk.FALSE)
        buttonbox.pack_start(self.close_button, expand = gtk.FALSE)
        buttonbox.show()

        self.shortcuts = gtk.CList(1, [ " Shortcuts " ])
        self.shortcuts.set_column_justification(0, gtk.JUSTIFY_CENTER)
        self.shortcuts.set_row_height(50)
        self.shortcuts.connect("select_row", self.__GnomeFileSelection_changeshortcut)
        self.__GnomeFileSelection_addshortcuts()
        self.shortcuts.show()

        rightbox = gtk.VBox(spacing = 5)
        rightbox.pack_start(filescrolledwin, expand = gtk.TRUE, fill = gtk.TRUE)
        rightbox.pack_start(filetable, expand = gtk.FALSE)
        rightbox.pack_start(buttonbox, expand = gtk.FALSE)
        rightbox.show()

        shortfilebox = gtk.HBox(spacing = 5)
        shortfilebox.pack_start(self.shortcuts, expand = gtk.FALSE)
        shortfilebox.pack_start(rightbox, expand = gtk.TRUE, fill = gtk.TRUE)
        shortfilebox.show()

        openfilebox = gtk.VBox(spacing = 5)
        openfilebox.pack_start(lookinbox, expand = gtk.FALSE)
        openfilebox.pack_start(shortfilebox)
        openfilebox.show()

        self.window.add(openfilebox)

    def __GnomeFileSelection_addshortcuts(self):
        for i in range(len(self.__GnomeFileSelection__shortcuts)):
            tuple = self.__GnomeFileSelection__shortcuts[i]
            self.shortcuts.append([tuple[1]])
            gdkcolor = hex2gdkcolor(self.window, "#7b7b7b")
            if (gdkcolor != None):
                self.shortcuts.set_background(i, gdkcolor)
            if (tuple[0] != "") and (self.icondir != ""):
                icon, mask = get_file_icon(tuple[0], self.window, self.icondir)
                self.shortcuts.set_pixmap(i, 0, icon, mask)

    def __GnomeFileSelection_getfreeshortcutindex(self):
        index = 0
        for i in range(len(self.__GnomeFileSelection__shortcuts)):
            if (self.__GnomeFileSelection__shortcuts[i][0] != ""):
                index = (i + 1)
            else:
                return index
        return index

    def __GnomeFileSelection_getlastshortcutindex(self):
        return self.__GnomeFileSelection_getfreeshortcutindex() - 1

    def __GnomeFileSelection_setshortcutbyindex(self, index, identifier, target):
        shortcut = self.__GnomeFileSelection__shortcuts[index]
        shortcut[0] = identifier
        target = sub("\$HOME", environ["HOME"], target)
        # Check wether the GNOME Desktop directory exists
        if (os.path.exists("%s/.gnome-desktop" % environ["HOME"])):
            target = sub("\$DESKTOP", "%s/.gnome-desktop" % environ["HOME"], target)
        # Check wether the KDE Desktop directory exists
        elif (os.path.exists("%s/Desktop" % environ["HOME"])):
            target = sub("\$DESKTOP", "%s/Desktop" % environ["HOME"], target)
        # Use the HOME directory
        else:
            target = sub("\$DESKTOP", environ["HOME"], target)
        target = sub("\$CWD", getcwd(), target)
        target = sub("\$LASTDIR", self.__GnomeFileSelection__lastdirectory, target)
        shortcut[1] = target
        self.__GnomeFileSelection__shortcuts[index] = shortcut
        if (identifier != "") and (self.icondir != ""):
            icon, mask = get_file_icon(identifier, self.window, self.icondir)
            self.shortcuts.set_pixmap(index, 0, icon, mask)

    def __GnomeFileSelection_readdirectory(self, filterchanged = gtk.FALSE):
        if (self.path == self.__GnomeFileSelection_lastdirectoryread) and (filterchanged == gtk.FALSE):
            return
        self.fileclist.clear()
        self.filename.set_text("")
        self.fileclist.freeze()
        if (self.log4py != None):
            self.log4py.debug("Reading directory: %s" % self.path)
        if (not (os.path.exists(self.path))):
            if (self.log4py != None):
                self.log4py.error("Error: Directory %s doesn't exist - using $HOME" % self.path)
            self.path = environ["HOME"]
            self.lookinpath.set_text(self.path)
        try:
            files = listdir(self.path)
        except (OSError), detail:
            files = []
            ErrorDialog("%s" % detail)
        files.sort()
        linenumber = 0

        # First show all directories
        for i in range(len(files)):
            filename = "%s/%s" % (self.path, files[i])
            showfile = not ((self.showhiddenfiles == gtk.FALSE) and (files[i][0] == "."))
            if (os.path.exists(filename)) and (os.path.isdir(filename)) and (showfile == gtk.TRUE):
                statinfo = stat(filename)
                if (self.icondir != ""):
                    self.fileclist.append(["", files[i], lng2str(statinfo[6]), asctime(localtime(statinfo[8]))])
                    icon, mask = get_file_icon("DIRECTORY", self.window, self.icondir)
                    self.fileclist.set_pixmap(linenumber, 0, icon, mask)
                else:
                    self.fileclist.append([files[i], lng2str(statinfo[6]), asctime(localtime(statinfo[8]))])
                if (divmod(linenumber, 2)[1] == 0):
                    gdkcolor = hex2gdkcolor(self.window, "#f6f6f6")
                    if (gdkcolor != None):
                        self.fileclist.set_background(linenumber, gdkcolor)
                linenumber = linenumber + 1

        # And now all other files and symlinks
        for i in range(len(files)):
            filename = "%s/%s" % (self.path, files[i])
            showfile = not ((self.showhiddenfiles == gtk.FALSE) and (files[i][0] == "."))
            if (showfile == gtk.TRUE):
                extensionok = gtk.FALSE
                for j in range(len(self.currentextension)):
                    extension = lower(self.currentextension[j])
                    if (extension == "*") or ((len(files[i]) >= len(extension)) and (extension == files[i][len(files[i])-(len(extension)):])):
                        extensionok = gtk.TRUE
            if (os.path.exists(filename)) and (not os.path.isdir(filename)) and (showfile == gtk.TRUE) and (extensionok == gtk.TRUE):
                statinfo = stat(filename)
                if (self.icondir != ""):
                    self.fileclist.append(["", files[i], lng2str(statinfo[6]), asctime(localtime(statinfo[8]))])
                    icon, mask = get_file_icon(files[i], self.window, self.icondir)
                    self.fileclist.set_pixmap(linenumber, 0, icon, mask)
                else:
                    self.fileclist.append([files[i], lng2str(statinfo[6]), asctime(localtime(statinfo[8]))])
                if (divmod(linenumber, 2)[1] == 0):
                    gdkcolor = hex2gdkcolor(self.window, "#f6f6f6")
                    if (gdkcolor != None):
                        self.fileclist.set_background(linenumber, gdkcolor)
                linenumber = linenumber + 1

        self.fileclist.thaw()
        self.__GnomeFileSelection_lastdirectoryread = self.path

    def __GnomeFileSelection_changetoparentdir(self, args):
        if (self.path != "/"):
            self.path = join(split(self.path, "/")[:-1], "/")
            if (self.path == ""):
                self.path = "/"
        self.lookinpath.set_text(self.path)
        self.__GnomeFileSelection_readdirectory()

    # Add a file by pressing Enter on the keyboard
    def __GnomeFileSelection_changedirectorykeyboard(self, clist, event):
        if ((event.keyval == 65293) or (event.keyval == 65421)):
            self.ok_button.emit("clicked")

    def __GnomeFileSelection_changedirectory(self, clist, event):
        if (event.type == gtk.gdk._2BUTTON_PRESS):               # Double click
            try:
                row = self.fileclist.get_selection_info(event.x, event.y)[0]
                if (self.icondir != ""):
                    columnnr = 1
                else:
                    columnnr = 0
                directory = self.fileclist.get_text(row, columnnr)
                if (self.path == "/"):
                    tmppath = "/%s" % directory
                else:
                    tmppath = "%s/%s" % (self.path, directory)
                if (os.path.isdir(tmppath)):
                    self.path = tmppath
                    self.lookinpath.set_text(self.path)
                    self.__GnomeFileSelection_readdirectory()
                else:
                    self.ok_button.emit("clicked")
            except:
                row = -1

    def __GnomeFileSelection_updatefileselection(self, clist, row, column, event):
        if (self.icondir != ""):
            columnnr = 1
        else:
            columnnr = 0
        if (len(self.fileclist.selection) == 1):
            filenames = self.fileclist.get_text(self.fileclist.selection[0], columnnr)
        else:
            filenames = ""
            for i in range(len(self.fileclist.selection)):
                filename = self.fileclist.get_text(self.fileclist.selection[i], columnnr)
                filenames = "%s\"%s\" " % (filenames, filename)
            if (len(filenames) > 0):
                filenames = filenames[:-1]
        self.filename.set_text(filenames)

    def __GnomeFileSelection_changedirectoryselection(self, clist, clistitem):
        path = self.lookinpath.get_text()
        if (os.path.isdir(path)):
            if (self.path != path):
                self.path = path
                self.__GnomeFileSelection_readdirectory()
        else:
            gnome.ui.ErrorDialog("Error - %s is not a valid directory !" % path)

    def __GnomeFileSelection_changedirectorymanually(self, entry, event):
        if ((event.keyval == 65293) or (event.keyval == 65421)):
            # path = self.lookinpath.gtk_entry().get_text()
            path = self.lookinpath.get_text()
            if (os.path.isdir(path)):
                if (self.path != path):
                    self.path = path
                    self.__GnomeFileSelection_readdirectory()
            else:
                gnome.ui.ErrorDialog("Error - %s is not a valid directory !" % path)

    def __GnomeFileSelection_changehiddenfiles(self, args):
        self.showhiddenfiles = self.hidden.get_active()
        self.__GnomeFileSelection_readdirectory(gtk.TRUE)

    def __GnomeFileSelection_activatefiletype(self, args):
        self.currentextension = self.filetypes[indexof(self.filetypeitems, self.filetypemenu.get_active())][1]
        if (self.__GnomeFileSelection__updateonactivate == gtk.TRUE):
            self.__GnomeFileSelection_readdirectory(gtk.TRUE)

    def __GnomeFileSelection_changeshortcut(self, clist, row, column, event):
        if (self.__GnomeFileSelection__shortcuts[row][1] != ""):
            self.path = self.__GnomeFileSelection__shortcuts[row][1]
            self.lookinpath.set_text(self.path)
            self.__GnomeFileSelection_readdirectory()

    def __GnomeFileSelection_loadlastdirectory(self):
        if (os.path.exists(self.__GnomeFileSelection__lastdirectoryfilename)):
            file = open(self.__GnomeFileSelection__lastdirectoryfilename, "r")
            self.__GnomeFileSelection__lastdirectory = strip(file.readline())
            file.close()
            if (self.log4py != None):
                self.log4py.debug("Last directory %s loaded from %s" % (self.__GnomeFileSelection__lastdirectory, self.__GnomeFileSelection__lastdirectoryfilename))
            self.path = self.__GnomeFileSelection__lastdirectory
            # FIXME: self.lookinpath.gtk_entry().set_text(self.__GnomeFileSelection__lastdirectory)
            self.lookinpath.set_text(self.__GnomeFileSelection__lastdirectory)

    def __GnomeFileSelection_savelastdirectory(self):
        file = open(self.__GnomeFileSelection__lastdirectoryfilename, "w")
        file.write("%s\n" % self.lookinpath.get_text())
        file.close()

    def show(self):
        self.__GnomeFileSelection_readdirectory()
        self.window.show()

    def destroy(self, args = None):
        if (self.__GnomeFileSelection__lastdirectoryfilename != ""):
            self.__GnomeFileSelection_savelastdirectory()
        self.window.destroy()

    def get_filenames(self):
        if (self.selectiontype == gtk.SELECTION_SINGLE):
            return self.filename.get_text()
        else:
            if (len(self.filename.get_text()) > 0):
                if (self.filename.get_text()[0] != "\""):
                    filenames = [self.filename.get_text()]
                else:
                    filenames = self.filename.get_text()
                    filenames = filenames[1:-1]
                    filenames = split(filenames, "\" \"")
            else:
                filenames = []
            return filenames

    def get_directory(self):
        return self.lookinpath.get_text()

    def clear_shortcuts(self):
        for i in range(len(self.__GnomeFileSelection__shortcuts)):
            if (self.__GnomeFileSelection__shortcuts[i][0] != ""):
                self.__GnomeFileSelection__shortcuts[i][0] = ""
                self.__GnomeFileSelection__shortcuts[i][1] = ""
                self.shortcuts.set_text(i, 0, "")

    def append_shortcut(self, identifier, target):
        freeindex = self.__GnomeFileSelection_getfreeshortcutindex()
        self.__GnomeFileSelection_setshortcutbyindex(freeindex, identifier, target)

    def remove_shortcut(self):
        lastindex = self.__GnomeFileSelection_getlastshortcutindex()
        self.__GnomeFileSelection__shortcuts[lastindex][0] = ""
        self.__GnomeFileSelection__shortcuts[lastindex][1] = ""
        self.shortcuts.set_text(lastindex, 0, "")

    def load_shortcuts_from_file(self, filename):
        self.clear_shortcuts()
        parser = ConfigParser()
        parser.read(filename)
        section = "shortcuts"
        for i in range(len(parser.options(section))):
            option = lower(parser.options(section)[i])
            if (option[:8] == "shortcut"):
                value = strip(parser.get(section, option, 1))
                if (value != ""):
                    if (find(value, ":") == -1):
                        if (self.log4py != None):
                            self.log4py.error("Invalid entry #%d in %s: %s" % (atoi(option[8:]), filename, value))
                    else:
                        splitted = split(value, ":")
                        index = atoi(option[8:])
                        self.__GnomeFileSelection_setshortcutbyindex(index, upper(strip(splitted[0])), strip(splitted[1]))

    def use_lastdirectory(self, filename):
        self.__GnomeFileSelection__lastdirectoryfilename = filename
        if (self.__GnomeFileSelection__lastdirectoryfilename != ""):
            self.__GnomeFileSelection_loadlastdirectory()

# ProgressBar: extendend progressbar class with additional information
class ProgressBar:

    def __init__(self, title, description, maxvalue, minvalue = 0.0):
        labDescription = gtk.Label(" %s\n" % description)
        labDescription.set_size_request(100, -1)
        labDescription.set_alignment(0, 0)
        labDescription.show()

        self.__GnomeProgressBar_value = minvalue
        self.__GnomeProgressBar_maxvalue = maxvalue

        self.progressbar = gtk.ProgressBar()
        self.progressbar.set_text("%d of %d (%2d%%)" % (self.__GnomeProgressBar_value, self.__GnomeProgressBar_maxvalue, self.__GnomeProgressBar_value / self.__GnomeProgressBar_maxvalue))
        self.progressbar.show()

        labTimeStart = gtk.Label(" Time started: ")
        labTimeStart.set_alignment(1, 0)
        labTimeStart.show()
        self.__GnomeProgressBar_starttime = time()
        self.__GnomeProgressBar_laststep = self.__GnomeProgressBar_starttime
        now = localtime(self.__GnomeProgressBar_starttime)
        self.timestart = gtk.Label("%s:%s:%s" % (zfill(now[3],2), zfill(now[4],2), zfill(now[5],2)))
        self.timestart.set_alignment(0, 0)
        self.timestart.show()
        labTimeEnd = gtk.Label(" Estimated end time: ")
        labTimeEnd.set_alignment(1, 0)
        labTimeEnd.show()
        self.timeend = gtk.Label("--:--:--")
        self.timeend.set_alignment(0, 0)
        self.timeend.show()
        labTimeDuration = gtk.Label(" Duration: ")
        labTimeDuration.set_alignment(1, 0)
        labTimeDuration.show()
        self.timeduration = gtk.Label("--:--:--.---")
        self.timeduration.set_alignment(0, 0)
        self.timeduration.show()
        labTimeRemaining = gtk.Label(" Time remaining: ")
        labTimeRemaining.set_alignment(1, 0)
        labTimeRemaining.show()
        self.timeremaining = gtk.Label("--:--:--.---")
        self.timeremaining.set_alignment(0, 0)
        self.timeremaining.show()

        statustable = gtk.Table(2, 5)                 # 2 rows, 5 columns
        statustable.attach(labTimeStart, 0, 1, 0, 1, gtk.FILL, 0, 1, 1)
        statustable.attach(self.timestart, 1, 2, 0, 1, 0, 0, 1, 1)
        statustable.attach(labTimeEnd, 0, 1, 1, 2, 0, 0, 1, 1)
        statustable.attach(self.timeend, 1, 2, 1, 2, 0, 0, 1, 1)
        statustable.attach(labTimeDuration, 2, 3, 0, 1, gtk.FILL, 0, 1, 1)
        statustable.attach(self.timeduration, 3, 4, 0, 1, 0, 0, 1, 1)
        statustable.attach(labTimeRemaining, 2, 3, 1, 2, 0, 0, 1, 1)
        statustable.attach(self.timeremaining, 3, 4, 1, 2, 0, 0, 1, 1)
        statustable.show()

        timingframe = gtk.Frame("Timing Information")
        timingframe.set_border_width(3)
        timingframe.add(statustable)
        timingframe.show()

        vbox = gtk.VBox()
        vbox.set_border_width(3)
        vbox.pack_start(labDescription)
        vbox.pack_start(timingframe)
        vbox.show()

        labThroughputCurrent = gtk.Label(" Current:")
        labThroughputCurrent.set_alignment(1, 0.5)
        labThroughputCurrent.show()
        self.throughput_current = gtk.Label(" ---")
        self.throughput_current.set_alignment(1, 0.5)
        self.throughput_current.show()
        labThroughputMinimum = gtk.Label(" Minimum:")
        labThroughputMinimum.set_alignment(1, 0.5)
        labThroughputMinimum.show()
        self.throughput_minimum = gtk.Label(" ---")
        self.throughput_minimum.set_alignment(1, 0.5)
        self.throughput_minimum.show()
        labThroughputAverage = gtk.Label(" Average:")
        labThroughputAverage.set_alignment(1, 0.5)
        labThroughputAverage.show()
        self.throughput_average = gtk.Label(" ---")
        self.throughput_average.set_alignment(1, 0.5)
        self.throughput_average.show()
        labThroughputMaximum = gtk.Label(" Maximum:")
        labThroughputMaximum.set_alignment(1, 0.5)
        labThroughputMaximum.show()
        self.throughput_maximum = gtk.Label(" ---")
        self.throughput_maximum.set_alignment(1, 0.5)
        self.throughput_maximum.show()

        self.__GnomeProgressBar_throughput_minimum = -1
        self.__GnomeProgressBar_throughput_maximum = -1

        throughputtable = gtk.Table(4, 2)
        throughputtable.attach(labThroughputCurrent, 0, 1, 0, 1, gtk.FILL, 0, 1, 1)
        throughputtable.attach(self.throughput_current, 1, 2, 0, 1)
        throughputtable.attach(labThroughputMinimum, 0, 1, 1, 2, gtk.FILL, 0, 1, 1)
        throughputtable.attach(self.throughput_minimum, 1, 2, 1, 2)
        throughputtable.attach(labThroughputAverage, 0, 1, 2, 3, gtk.FILL, 0, 1, 1)
        throughputtable.attach(self.throughput_average, 1, 2, 2, 3)
        throughputtable.attach(labThroughputMaximum, 0, 1, 3, 4, gtk.FILL, 0, 1, 1)
        throughputtable.attach(self.throughput_maximum, 1, 2, 3, 4)
        throughputtable.show()

        throughputframe = gtk.Frame("Throughput / Min")
        throughputframe.set_border_width(3)
        throughputframe.add(throughputtable)
        throughputframe.show()

        hbox = gtk.HBox()
        hbox.set_border_width(3)
        hbox.pack_start(vbox)
        hbox.pack_start(throughputframe)
        hbox.show()

        vboxprogress = gtk.VBox()
        vboxprogress.set_border_width(3)
        vboxprogress.pack_start(hbox)
        vboxprogress.pack_start(self.progressbar)
        vboxprogress.show()

        self.window = gtk.Window()
        self.window.set_title(title)
        self.window.set_border_width(3)
        self.window.add(vboxprogress)
        self.window.show()

        self.redraw()

    def redraw(self):
        # FIXME: make sure window is being drawn; use gtk.idle_add probably
        while (gtk.events_pending()):
            gtk.mainiteration(0)

    def update(self, step = 1):
        self.redraw()
        self.__GnomeProgressBar_value = self.__GnomeProgressBar_value + step
        self.progressbar.set_fraction(self.__GnomeProgressBar_value / self.__GnomeProgressBar_maxvalue)
        self.progressbar.set_text("%d of %d (%2d%%)" % (self.__GnomeProgressBar_value, self.__GnomeProgressBar_maxvalue, self.progressbar.get_fraction() * 100))
        currenttime = time()
        duration = currenttime - self.__GnomeProgressBar_starttime
        durationlaststep = currenttime - self.__GnomeProgressBar_laststep
        self.__GnomeProgressBar_laststep = currenttime
        throughput_current = (60 / durationlaststep) / step
        if ((self.__GnomeProgressBar_throughput_minimum == -1) or (throughput_current < self.__GnomeProgressBar_throughput_minimum)):
            self.__GnomeProgressBar_throughput_minimum = throughput_current
            self.throughput_minimum.set_text(" %.2f " % self.__GnomeProgressBar_throughput_minimum)
        if ((self.__GnomeProgressBar_throughput_maximum == -1) or (throughput_current > self.__GnomeProgressBar_throughput_maximum)):
            self.__GnomeProgressBar_throughput_maximum = throughput_current
            self.throughput_maximum.set_text(" %.2f " % self.__GnomeProgressBar_throughput_maximum)
        percent = self.progressbar.get_fraction()
        remaining = (duration / percent) * (1 - percent)
        endtime = localtime(self.__GnomeProgressBar_starttime + duration + remaining)
        self.timeduration.set_text("%s " % format_duration(duration))
        self.timeremaining.set_text("%s " % format_duration(remaining))
        self.timeend.set_text("%s:%s:%s" % (zfill(endtime[3],2), zfill(endtime[4],2), zfill(endtime[5],2)))
        self.throughput_current.set_text(" %.2f " % throughput_current)
        self.throughput_average.set_text(" %.2f " % ((60 / duration) * self.__GnomeProgressBar_value))
        self.redraw()

    def close(self):
        sleep(0.3)
        self.window.destroy()
        self.window = None

# Pixmap: easier, faster and more flexible way to access pixmaps
class Pixmap:

    def __init__(self, window, pixmap, label = None):
        icon, mask = create_pixmap(window, pixmap)
        self.__Pixmap_pixmap = gtk.Image()
        self.__Pixmap_pixmap.set_from_pixmap(icon, mask)
        self.__Pixmap_pixmap.show()
        if (label != None):
            self.__Pixmap_label = gtk.Label(label)
            self.__Pixmap_label.set_padding(2, 2)
            self.__Pixmap_label.show()
            self.__Pixmap_hbox = gtk.HBox()
            self.__Pixmap_hbox.add(self.__Pixmap_pixmap)
            self.__Pixmap_hbox.add(self.__Pixmap_label)
            self.__Pixmap_hbox.show()
            self.__Pixmap_object = self.__Pixmap_hbox
        else:
            self.__Pixmap_label = None
            self.__Pixmap_hbox = None
            self.__Pixmap_object = self.__Pixmap_pixmap

    def show(self):
        self.__Pixmap_object.show()

    def hide(self):
        self.__Pixmap_object.hide()

    def get_pixmap(self):
        return self.__Pixmap_pixmap

    def get_hbox(self):
        return self.__Pixmap_hbox

# OptionMenu is an easy way to build OptionMenus
class OptionMenu:

    def __init__(self, loglevel = 1 << 1):
        self.__GnomeOptionMenu_optionmenu = gtk.OptionMenu()
        self.__GnomeOptionMenu_menu = gtk.Menu()
        self.__GnomeOptionMenu_menu.show()
        self.__GnomeOptionMenu_previousitem = None
        self.__GnomeOptionMenu_items = {}
        if (log4pyavailable == gtk.TRUE):
            self.log4py = Logger().get_instance(self)
            self.log4py.set_loglevel(loglevel)
        else:
            self.log4py = None

    def get_active_item(self):
        for i in range(len(self.__GnomeOptionMenu_items.keys())):
            key = self.__GnomeOptionMenu_items.keys()[i]
            if (self.__GnomeOptionMenu_items[key].active):
                return key
        return None

    def activate_item(self, key):
        if (self.__GnomeOptionMenu_items.has_key(key)):
            item = self.__GnomeOptionMenu_items[key]
            self.__GnomeOptionMenu_menu.select_item(item)
            self.__GnomeOptionMenu_menu.activate_item(item, gtk.TRUE)
        else:
            if (self.log4py != None):
                self.log4py.error("Item %s doesn't exist." % key)

    def get_gtkoptionmenu(self):
        return self.__GnomeOptionMenu_optionmenu

    def append_item(self, item):
        self.__GnomeOptionMenu_items[item] = gtk.RadioMenuItem(self.__GnomeOptionMenu_previousitem, item)
        self.__GnomeOptionMenu_items[item].show()
        self.__GnomeOptionMenu_menu.append(self.__GnomeOptionMenu_items[item])
        self.__GnomeOptionMenu_previousitem = self.__GnomeOptionMenu_items[item]

    def append_itemlist(self, itemlist):
        for i in range(len(itemlist)):
            self.append_item(itemlist[i])

    def show(self):
        self.__GnomeOptionMenu_optionmenu.set_menu(self.__GnomeOptionMenu_menu)
        self.__GnomeOptionMenu_optionmenu.show()

class TipWindow:

    def __init__(self, config_file, tips, icondir, application_name, boolshowtipalways = gtk.FALSE):
        """ Initialize the window and show a random tip. """

        self.__GnomeTip_current_tip = -1
        self.__GnomeTip_config_file = config_file
        self.__GnomeTip_load_config()
        if (self.__GnomeTip_show_window == gtk.FALSE) and (boolshowtipalways == gtk.FALSE):
            self.window = None
            return                                # Somehow the user doesn't like tips ...
        self.__GnomeTip_all_tips = tips
        self.window = gtk.Window()
        if (application_name != ""):
            self.window.set_title("%s - Tip of the Day" % application_name)
        else:
            self.window.set_title("Tip of the Day")
        self.window.connect("destroy", self.__GnomeTip_close)

        self.tip = gtk.Label("")
        self.__GnomeTip_set_tip(self.__GnomeTip_get_random_tip())
        self.tip.show()

        tipBox = gtk.HBox(spacing = 5)
        if (os.path.exists("%s/gnome-hint.xpm" % icondir)):
            icon, mask = create_pixmap(self.window, "%s/gnome-hint.xpm" % icondir)
            pixmap = gtk.Image()
            pixmap.set_from_pixmap(icon, mask)
            pixmap.show()
            tipBox.pack_start(pixmap, expand = gtk.FALSE)
        tipBox.pack_start(self.tip)
        tipBox.show()

        closeButton = gtk.Button(stock="gtk-close")
        closeButton.connect("clicked", self.__GnomeTip_close)
        closeButton.show()
        nextTipButton = gtk.Button(" Next Tip ")
        nextTipButton.connect("clicked", self.__GnomeTip_next_tip)
        nextTipButton.show()
        previousTipButton = gtk.Button(" Previous Tip ")
        previousTipButton.connect("clicked", self.__GnomeTip_previous_tip)
        previousTipButton.show()

        self.showTipOnStartUp = gtk.CheckButton("Show tips on startup")
        self.showTipOnStartUp.set_active(self.__GnomeTip_show_window)
        self.showTipOnStartUp.show()

        dummyLabel = gtk.Label("")
        dummyLabel.show()
        
        buttonBar = gtk.HBox(spacing = 3)
        buttonBar.pack_start(self.showTipOnStartUp, expand = gtk.FALSE)
        buttonBar.pack_start(dummyLabel)
        buttonBar.pack_start(previousTipButton, expand = gtk.FALSE)
        buttonBar.pack_start(nextTipButton, expand = gtk.FALSE)
        buttonBar.pack_start(closeButton, expand = gtk.FALSE)
        buttonBar.show()

        fullBox = gtk.VBox()
        fullBox.pack_start(tipBox, expand = gtk.FALSE)
        fullBox.pack_start(buttonBar, expand = gtk.FALSE)
        fullBox.show()

        self.window.set_border_width(3)
        self.window.add(fullBox)

    def __GnomeTip_set_tip(self, tip):
        """ Set a new tip. """
        self.__GnomeTip_current_tip = self.__GnomeTip_all_tips.index(tip)
        text = "Tip %d: %s" % (self.__GnomeTip_current_tip + 1, tip)
        self.tip.set_text(text)

    def __GnomeTip_next_tip(self, args):
        """ Move on to the next tip. """
        if (len(self.__GnomeTip_all_tips) >= (self.__GnomeTip_current_tip + 2)):
            self.__GnomeTip_set_tip(self.__GnomeTip_all_tips[self.__GnomeTip_current_tip + 1])
        else:
            self.__GnomeTip_set_tip(self.__GnomeTip_all_tips[0])

    def __GnomeTip_previous_tip(self, args):
        """ Show the previous tip. """
        if (self.__GnomeTip_current_tip > 0):
            self.__GnomeTip_set_tip(self.__GnomeTip_all_tips[self.__GnomeTip_current_tip - 1])
        else:
            self.__GnomeTip_set_tip(self.__GnomeTip_all_tips[len(self.__GnomeTip_all_tips) - 1])

    def __GnomeTip_close(self, args):
        """ Close the GnomeTip window. """
        self.__GnomeTip_save_config()
        self.window.destroy()

    def __GnomeTip_get_random_tip(self):
        """ Returns a random tip (for startup). """
        tip = choice(self.__GnomeTip_all_tips)
        return tip

    def __GnomeTip_load_config(self):
        """ Load the configuration file. """
        if (os.path.exists(self.__GnomeTip_config_file)):
            parser = ConfigParser()
            parser.read(self.__GnomeTip_config_file)
            self.__GnomeTip_show_window = parser.getboolean("GnomeTip", "ShowGnomeTipWindow")
        else:
            self.__GnomeTip_show_window = gtk.TRUE

    def __GnomeTip_save_config(self):
        """ Save the configuration file. """
        file = open(self.__GnomeTip_config_file, "w")
        file.write("[GnomeTip]\n")
        file.write("ShowGnomeTipWindow: %d\n" % self.showTipOnStartUp.get_active())
        file.close()

    def show(self):
        """ Show the GnomeTip window. """
        if (self.window != None):
            self.window.show()

class ErrorDialog:

    def __init__(self, message):
        dialog = gtk.MessageDialog(None, 
                                   gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                                   gtk.MESSAGE_ERROR, gtk.BUTTONS_OK,
                                   message)
        dialog.run()
        dialog.destroy()

class InfoDialog:

    def __init__(self, message):
        dialog = gtk.MessageDialog(None, 
                                   gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                                   gtk.MESSAGE_INFO, gtk.BUTTONS_OK,
                                   message)
        dialog.run()
        dialog.destroy()

class WarningDialog:

    def __init__(self, message):
        dialog = gtk.MessageDialog(None, 
                                   gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                                   gtk.MESSAGE_WARNING, gtk.BUTTONS_OK,
                                   message)
        dialog.run()
        dialog.destroy()

class QuestionDialog:

    def __init__(self, message, buttons_type = gtk.BUTTONS_YES_NO):
        dialog = gtk.MessageDialog(None, 
                                   gtk.DIALOG_DESTROY_WITH_PARENT,
                                   gtk.MESSAGE_QUESTION, buttons_type,
                                   message)
        self.__QuestionDialog_result = dialog.run()
        dialog.destroy()

    def get_result(self):
        return self.__QuestionDialog_result

class InputDialog:

    def __init__(self, parent, title, message, text = "", width = -1):
        label = gtk.Label(" %s " % message)
        label.show()
        entry = gtk.Entry()
        entry.set_size_request(width, -1)
        entry.set_text(text)
        entry.show()
        hbox = gtk.HBox()
        hbox.set_border_width(3)
        hbox.pack_start(label)
        hbox.pack_start(entry)
        hbox.show()
        buttons = (gtk.STOCK_OK, gtk.RESPONSE_OK, gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
        dialog = gtk.Dialog(title, parent, gtk.DIALOG_DESTROY_WITH_PARENT or gtk.DIALOG_MODAL, buttons)
        dialog.set_default_response(gtk.RESPONSE_OK)
        dialog.vbox.pack_start(hbox)
        dialog.show()
        self.__InputDialog_result = dialog.run()
        self.__InputDialog_text = entry.get_text()
        dialog.destroy()

    def get_result(self):
        if (self.__InputDialog_result == gtk.RESPONSE_OK):
            return self.__InputDialog_text
        else:
            return ""
